# -*- coding: utf-8 -*-

#   Didacto, un logiciel d'aide à l'organisation d'un corpus didactique
#   Copyright (C) 2020  Marco de Freitas
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.
#   If not, see <https://www.gnu.org/licenses/>.
#
#   contact:marco@sillage.ch

import os

from PyPDF2 import PdfFileReader, utils


def sorting(thing):
    """Sort the list of dictionnary by lowered key values, alphabetic order.
    """

    def my_lower(thung):
        return thung['name'].lower()

    for item in sorted(thing):
        thing[item].sort(key=my_lower)

    return thing


def get_keywords_from_field(field, separator):
    """This function takes a keywords field and separator, keywords values or ∅ if empty"""

    if field is None:
        newKeywords = "∅"
    else:
        newKeywords = field.split(separator)

    return newKeywords


class Model:
    def __init__(self, prefs, user_data):
        """Initialise prefs values retrieved by controller."""
        self.user_data = UserData(user_data)
        self.errors_text = ''
        path = prefs['path']

        self.separator = prefs['separator']
        if self.separator == '':
            self.separator = None

        notation = prefs['notation_format']

    wordsDict = {}
    path = ''
    emptyDict = {}

    # Sous-fonctions de scan_repertory()
    def get_info(self, path):
        """This function takes a pdf file path and returns pdf metadata keywords field"""
        errors = []
        with open(path, 'rb') as f:
            try:
                pdf = PdfFileReader(f, strict=False)
            except ValueError:
                errors.append("problem with file:" + path + "\n")
                return None
            info = pdf.getDocumentInfo()
            temp = {}
            if info is not None:
                for key in info:
                    temp[key] = info[key]
                    try:
                        key = str(temp["/Keywords"])
                    except KeyError:  # field is empty string ''
                        key = '∅'
                    finally:
                        if not key:  # field is empty
                            key = '∅'
            else:
                key = '∅'
        self.errors_text = " ".join(iter(errors))
        return key

    # Cette fonction parcours l'arborescence à partir du dossier sélectionné
    # et récupère les mots clés
    def scan_repertory(self, path, recursive):
        """This function scan a directory for pdf file keywords and store the results in a custom Dictionary:
        { keyword1 : [{'name' : 'file1name.pdf', 'path' : '/file1/path'},
                      {'name' : 'file2name.pdf', 'path' : '/file2/path'},
                      {'name' : 'file3name.pdf', 'path' : '/file3/path'}]}

        Keywords and file names are sorted by alpahbetic order

        Good to know:
        os.walk method retrieves a generator object where each item a tuple
        ('folder path', ['subfolder1', 'subfolder2', etc.], ['file1name', 'file2name', etc.])
        """
        keywordsDict = {}
        pdfKey = ''
        errors = []
        if recursive is True:
            for i in os.walk(path, topdown=True):
                for file_name in i[2]:
                    if file_name.endswith('.pdf'):
                        try:
                            pdfKey = self.get_info(i[0] + '/' + file_name)
                        except FileNotFoundError:
                            errors.append("File does not exist:" + i[0] + '/' + file_name + "\n")
                        except utils.PdfReadError:
                            errors.append("problem with file:" + i[0] + '/' + file_name + "\n")
                            # continue
                        else:
                            keywords = (get_keywords_from_field(pdfKey, self.separator))
                            if keywords is not None:
                                for word in keywords:
                                    if word not in keywordsDict:
                                        keywordsDict[word] = [{"name": file_name,
                                                               "path": i[0]}]
                                    else:
                                        keywordsDict[word].append({"name": file_name,
                                                                   "path": i[0]})
        if recursive is False:
            for name in next(os.walk(path, topdown=True))[2]:
                if name.endswith('.pdf'):
                    try:
                        pdfKey = self.get_info(path + '/' + name)
                    except FileNotFoundError:
                        errors.append("File does not exist:" + path + '/' + name + "\n")
                    except utils.PdfReadError:
                        errors.append("Problem with file:" + path + '/' + name + "\n")
                    else:
                        keywords = (get_keywords_from_field(pdfKey, self.separator))
                        if keywords is not None:
                            for word in keywords:
                                if word not in keywordsDict:
                                    keywordsDict[word] = [{"name": name,
                                                           "path": path}]
                                else:
                                    keywordsDict[word].append({"name": name,
                                                               "path": path})
        self.wordsDict = sorting(keywordsDict)
        self.errors_text = " ".join(iter(errors))  # lists are iterable but not iterators
        # print(self.wordsDict.keys())


class UserData:
    """This Class stores user data, keyword definition and categories
    curent {'keyword1':'definition1', etc.}
    future {'keyword1':{'definition':'definition1',
                        'categories':['cat1','cat2', etc.]
                        }}
    simpler {'keyword1':'definition1',['cat1','cat2','cat3']}

    even simpler, two different dicts, in two files
    or go for sql lite ???
    """

    def __init__(self, data, ):
        self.user_data = data

    def get(self, key):
        return self.user_data[key]

    def set(self, key, data):
        self.user_data[key] = data

    # new methods to handle keywords categories
    def get_categories(self, key):
        return self.user_data[key][1]

    def set_categries(self, key, cat):
        self.user_data[key][1] = cat

    def retrieve(self):
        return self.user_data
